@@ -13,7 +13,7 @@ docvars(myCorpus, "Year") <- as.integer(substring(names(inaugTexts), 1, 4))
 summary(myCorpus, n=5)
 
 ## ------------------------------------------------------------------------
-language(myCorpus) <- "english"
+metadoc(myCorpus, "language") <- "english"
 metadoc(myCorpus, "docsource")  <- paste("inaugTexts", 1:ndoc(myCorpus), sep="_")
 summary(myCorpus, n=5, showmeta=TRUE)
 
